from flask.ext.security import UserMixin, RoleMixin

from extensions import db
from core.models import CRUDMixin


roles_users = db.Table('roles_user',
    db.Column('user_id', db.Integer, db.ForeignKey('users.id')),
    db.Column('role_id', db.Integer, db.ForeignKey('roles.id'))
)


class User(db.Model, CRUDMixin, UserMixin):
    email = db.Column(db.String(255), unique=True)
    password = db.Column(db.String(255))
    active = db.Column(db.Boolean, default=False)
    confirmed_at = db.Column(db.DateTime)
    roles = db.relationship('Role', secondary=roles_users,
                            backref=db.backref('users', lazy='dynamic'))

    def __repr__(self):
        return "<{}: {}>".format(self.__class__.__name__, self.email)


class Role(db.Model, CRUDMixin, RoleMixin):
    name = db.Column(db.String(80), unique=True)
    description = db.Column(db.UnicodeText)
