from datetime import datetime
from sqlalchemy.sql.expression import desc

from extensions import db
from core.models import SlugMixin


class BlogPost(db.Model, SlugMixin):

    content = db.Column(db.UnicodeText)
    author_id = db.Column(db.Integer, db.ForeignKey('users.id'),
                          nullable=False)
    author = db.relationship('User')
    updated_at = db.Column(db.DateTime, default=datetime.utcnow,
                           onupdate=datetime.utcnow)

    discriminator = db.Column('type', db.String(50))
    __mapper_args__ = {'polymorphic_on': discriminator}

    @classmethod
    def latest(cls, limit=20):
        """ Return query for latest posts
        """
        q_ordered = cls.query.order_by(desc(cls.created_at))
        return q_ordered.limit(limit).offset(0)

    @classmethod
    def find_by_slug(cls, slug):
        return cls.query.filter_by(slug=slug).first_or_404()


class ImagePost(BlogPost):

    image = db.Column(db.String(256))

    __tablename__ = BlogPost.__tablename__
    __mapper_args__ = {'polymorphic_identity': 'image_post'}

    def add_image(self, image_path):
        self.image = image_path
