from __future__ import absolute_import
from flask import g, render_template, request, redirect, url_for
from flask.ext.uploads import IMAGES, UploadSet

from .forms import PostForm
from .models import BlogPost, ImagePost


posters = UploadSet('POSTERS', IMAGES)


def show_posts():
    posts = BlogPost.latest()
    return render_template('blog/list_posts.html', posts=posts)


def show_post(slug):
    post = BlogPost.find_by_slug(slug)
    return render_template('blog/show_post.html', post=post)


def add_post():
    post_form = PostForm(formdata=request.form or None)

    if request.form and post_form.validate():
        if 'poster' in request.files:
            filename = posters.save(request.files['poster'])
            blog_post = ImagePost(image=filename)
        else:
            blog_post = BlogPost()

        post_form.populate_obj(blog_post)
        blog_post.author = g.identity.user
        blog_post.save()
        return redirect(url_for('blog.show_post', slug=blog_post.slug))

    return render_template('blog/add_post.html', post_form=post_form)


def edit_post():
    return render_template('edit_post.html')
