from __future__ import absolute_import
import os
from flask import Flask, request
from extensions import register_jinja_helpers
from werkzeug.utils import import_string


class ExtensionLoadError(Exception):
    pass


class BlueprintLoadError(Exception):
    pass


class AppFactory(object):

    def __init__(self, config, envvar='PROJECT_SETTINGS', bind_db_object=True):
        self.app_config = config
        self.app_envvar = os.environ.get(envvar, False)
        # self.bind_db_object = bind_db_object

    def init_app(self, app_name, **kwargs):
        app = Flask(app_name, **kwargs)
        app.config.from_object(self.app_config)
        app.config.from_envvar(self.app_envvar, silent=True)

        self._bind_extensions(app)
        self._register_blueprints(app)

        return app

    def _import(self, path):
        module_name, object_name = path.rsplit('.', 1)
        module = import_string(module_name)
        return module, object_name

    def _bind_extensions(self, app):
        for ext_path in app.config.get('EXTENSIONS', []):
            module, ext_name = self._import(ext_path)

            try:

                ext = getattr(module, ext_name)
                if ext_name == 'babel':
                    register_jinja_helpers(app)

                    @ext.localeselector
                    def get_locale():
                        return request.accept_languages.best_match(['ru', 'en'])

            except AttributeError:
                ExtensionLoadError("Extension '{}'' not found".format(ext))

            try:
                ext.init_app(app)
            except AttributeError:
                ext(app)

    def _register_blueprints(self, app):
        for blueprint_path in app.config.get('BLUEPRINTS', []):
            module, bp_name = self._import(blueprint_path)
            if hasattr(module, bp_name):
                app.register_blueprint(getattr(module, bp_name))
            else:
                raise BlueprintLoadError('No {} blueprint found'.format(bp_name))
